/*!
 * (C) PAQT.com B.V. https://paqt.com - MIT License
 */
import { proxyCustomElement, HTMLElement, h } from '@stencil/core/internal/client';
import { h as hasSlot } from './slot.js';

const fieldGroupCss = ".gr-scroll-lock{overflow:hidden !important}:host{--margin-between-fields:var(--gr-spacing-xx-small);--margin-between-fields-horizontal:var(--gr-spacing-medium);--align-fields:flex-start;display:block;box-sizing:border-box}:host *,:host *:before,:host *:after{box-sizing:inherit}.field-group{margin:0;padding:0;border:none}.content{display:flex;flex-direction:column;align-items:var(--align-fields);gap:var(--margin-between-fields)}.field-group-horizontal .content{flex-direction:row;gap:var(--margin-between-fields-horizontal)}.field-group-label{display:none}.field-group-has-label .field-group-label{line-height:var(--gr-line-height-normal);color:var(--gr-form-element-label-color);margin-bottom:var(--gr-spacing-xxx-small);font-size:var(--gr-form-element-label-font-size-medium);margin:0;padding:0;display:flex;margin-bottom:var(--gr-spacing-xxx-small)}";

const FieldGroup = /*@__PURE__*/ proxyCustomElement(class extends HTMLElement {
  constructor() {
    super();
    this.__registerHost();
    this.__attachShadow();
    this.hasLabelSlot = false;
    this.label = '';
    this.horizontal = false;
  }
  handleLabelChange() {
    this.handleSlotChange();
  }
  connectedCallback() {
    this.handleSlotChange = this.handleSlotChange.bind(this);
    this.el.shadowRoot.addEventListener('slotchange', this.handleSlotChange);
  }
  componentWillLoad() {
    this.handleSlotChange();
  }
  disconnectedCallback() {
    this.el.shadowRoot.removeEventListener('slotchange', this.handleSlotChange);
  }
  handleSlotChange() {
    this.hasLabelSlot = hasSlot(this.el, 'label');
  }
  render() {
    const hasLabel = this.label ? true : this.hasLabelSlot;
    return (h("fieldset", { class: {
        'field-group': true,
        'field-group-horizontal': this.horizontal,
        'field-group-has-label': hasLabel,
      } }, h("legend", { class: "field-group-label", "aria-hidden": hasLabel ? 'false' : 'true' }, h("slot", { name: "label" }, this.label)), h("div", { class: "content" }, h("slot", null))));
  }
  get el() { return this; }
  static get watchers() { return {
    "label": ["handleLabelChange"]
  }; }
  static get style() { return fieldGroupCss; }
}, [1, "gr-field-group", {
    "label": [1],
    "horizontal": [516],
    "hasLabelSlot": [32]
  }]);
function defineCustomElement$1() {
  if (typeof customElements === "undefined") {
    return;
  }
  const components = ["gr-field-group"];
  components.forEach(tagName => { switch (tagName) {
    case "gr-field-group":
      if (!customElements.get(tagName)) {
        customElements.define(tagName, FieldGroup);
      }
      break;
  } });
}
defineCustomElement$1();

const GrFieldGroup = FieldGroup;
const defineCustomElement = defineCustomElement$1;

export { GrFieldGroup, defineCustomElement };

//# sourceMappingURL=gr-field-group.js.map