/*!
 * (C) PAQT.com B.V. https://paqt.com - MIT License
 */
import { proxyCustomElement, HTMLElement, createEvent, h, Host } from '@stencil/core/internal/client';
import { a as addEventListener, i as inheritAttributes, b as removeEventListener } from './helpers.js';

const radioCss = ".gr-scroll-lock{overflow:hidden !important}:host{--label-color:var(--gr-color-dark-tint);--label-margin-top-adjustment:-7px;--label-margin-bottom-adjustment:-4px;--control-size:var(--gr-toggle-size);--control-border-color:var(--gr-color-light-shade);--control-background-color:var(--gr-color-white);--control-border-color-hover:var(--gr-color-medium);--control-background-color-hover:var(--gr-color-white);--control-border-color-focus:var(--gr-color-medium);--control-background-color-focus:var(--gr-color-medium);--control-color-checked:var(--gr-color-white);--control-border-color-checked:var(--gr-color-primary);--control-background-color-checked:var(--gr-color-primary);--control-border-color-checked-hover:var(--gr-color-primary-shade);--control-background-color-checked-hover:var(--gr-color-primary-shade);--control-border-color-checked-focus:var(--gr-color-primary-shade);--control-background-color-checked-focus:var(--gr-color-primary-shade);--focus-ring:0 0 0 var(--gr-focus-ring-width) rgb(var(--gr-color-primary-rgb), 0.33);display:inline-block;box-sizing:border-box}:host *,:host *:before,:host *:after{box-sizing:inherit}:host(.radio-disabled){pointer-events:none}.radio-top{align-items:flex-start}.radio-center{align-items:center}.radio-bottom{align-items:flex-end}.radio{display:inline-flex;font-family:var(--gr-font-family);font-size:var(--gr-form-element-font-size-medium);font-weight:var(--gr-font-weight-normal);color:var(--label-color);vertical-align:middle;cursor:pointer}.radio-icon{display:inline-flex;width:var(--control-size);height:var(--control-size)}.radio-icon svg{width:100%;height:100%}.radio-control{position:relative;display:inline-flex;align-items:center;justify-content:center;width:var(--control-size);height:var(--control-size);border:solid 1px var(--control-border-color);border-radius:50%;background-color:var(--control-background-color);color:transparent;transition:150ms border-color, 150ms background-color, 150ms color, 150ms box-shadow}.radio-control input[type=radio]{position:absolute;opacity:0;padding:0;margin:0;pointer-events:none}.radio:not(.radio-checked):not(.radio-disabled) .radio-control:hover{border-color:var(--control-border-color-hover);background-color:var(--control-background-color-hover)}.radio.radio-focused:not(.radio-checked):not(.radio-disabled) .radio-control{border-color:var(--control-border-color-hover);background-color:var(--control-background-color-hover);box-shadow:var(--focus-ring)}.radio-checked .radio-control{color:var(--control-color-checked);border-color:var(--control-border-color-checked);background-color:var(--control-background-color-checked)}.radio.radio-checked:not(.radio-disabled) .radio-control:hover{border-color:var(--control-border-color-checked-hover);background-color:var(--control-background-color-checked-hover)}.radio.radio-checked:not(.radio-disabled).radio-focused .radio-control{border-color:var(--control-border-color-checked-focus);background-color:var(--control-background-color-checked-focus);box-shadow:var(--focus-ring)}.radio-disabled{opacity:0.5}.radio-label{line-height:var(--gr-line-height-normal);margin-left:0.5em;user-select:none}.radio-top .radio-label{margin-top:var(--label-margin-top-adjustment)}.radio-bottom .radio-label{margin-bottom:var(--label-margin-bottom-adjustment)}:host(:focus){outline:none}";

let id = 0;
const Radio = /*@__PURE__*/ proxyCustomElement(class extends HTMLElement {
  constructor() {
    super();
    this.__registerHost();
    this.__attachShadow();
    this.grBlur = createEvent(this, "gr-blur", 7);
    this.grFocus = createEvent(this, "gr-focus", 7);
    this.inputId = `radio-${++id}`;
    this.labelId = `radio-label-${id}`;
    this.radioGroup = null;
    this.inheritedAttributes = {};
    this.updateState = () => {
      if (this.radioGroup) {
        this.checked = this.radioGroup.value === this.value;
      }
    };
    this.hasFocus = false;
    this.value = undefined;
    this.disabled = false;
    this.checked = false;
    this.alignment = 'center';
    this.buttonTabindex = -1;
  }
  /** @internal */
  async setButtonTabindex(value) {
    this.buttonTabindex = value;
  }
  connectedCallback() {
    this.handleBlur = this.handleBlur.bind(this);
    this.handleFocus = this.handleFocus.bind(this);
    if (this.value === undefined) {
      this.value = this.inputId;
    }
    const radioGroup = (this.radioGroup = this.el.closest('gr-radio-group'));
    if (radioGroup) {
      this.updateState();
      addEventListener(radioGroup, 'gr-change', this.updateState);
      // Needed for the Vue wrappers
      addEventListener(radioGroup, 'v-gr-change', this.updateState);
    }
  }
  componentWillLoad() {
    this.inheritedAttributes = inheritAttributes(this.el, ['aria-label']);
  }
  disconnectedCallback() {
    const radioGroup = this.radioGroup;
    if (radioGroup) {
      removeEventListener(radioGroup, 'gr-change', this.updateState);
      // Needed for the Vue wrappers
      removeEventListener(radioGroup, 'v-gr-change', this.updateState);
      this.radioGroup = null;
    }
  }
  /** Sets focus on the radio. */
  async setFocus(options) {
    this.input.focus(options);
  }
  /** Removes focus from the radio. */
  async removeFocus() {
    this.input.blur();
  }
  handleBlur() {
    this.hasFocus = false;
    this.grBlur.emit();
  }
  handleFocus() {
    this.hasFocus = true;
    this.grFocus.emit();
  }
  render() {
    const ariaLabelAttributes = this.inheritedAttributes['aria-label']
      ? { 'aria-label': this.inheritedAttributes['aria-label'] }
      : { 'aria-labelledby': this.labelId };
    return (h(Host, Object.assign({ class: {
        'radio-disabled': this.disabled,
      }, tabindex: this.buttonTabindex, role: "radio", onBlur: this.handleBlur, onFocus: this.handleFocus, "aria-checked": `${this.checked}`, "aria-hidden": this.disabled ? 'true' : null }, ariaLabelAttributes), h("label", { class: {
        'radio': true,
        'radio-checked': this.checked,
        'radio-disabled': this.disabled,
        'radio-focused': this.hasFocus,
        [`radio-${this.alignment}`]: true,
      }, htmlFor: this.inputId }, h("span", { class: "radio-control" }, h("span", { class: "radio-icon" }, h("svg", { role: "img", "aria-hidden": "true", viewBox: "0 0 16 16" }, h("g", { stroke: "none", "stroke-width": "1", fill: "none", "fill-rule": "evenodd" }, h("g", { fill: "currentColor" }, h("circle", { cx: "8", cy: "8", r: "3.42857143" }))))), h("input", { ref: el => (this.input = el), id: this.inputId, type: "radio", value: this.value, checked: this.checked, disabled: this.disabled, tabindex: "-1" })), h("span", { id: this.labelId, class: "radio-label" }, h("slot", null)))));
  }
  get el() { return this; }
  static get style() { return radioCss; }
}, [1, "gr-radio", {
    "value": [1537],
    "disabled": [4],
    "checked": [1540],
    "alignment": [513],
    "hasFocus": [32],
    "buttonTabindex": [32],
    "setButtonTabindex": [64],
    "setFocus": [64],
    "removeFocus": [64]
  }]);
function defineCustomElement$1() {
  if (typeof customElements === "undefined") {
    return;
  }
  const components = ["gr-radio"];
  components.forEach(tagName => { switch (tagName) {
    case "gr-radio":
      if (!customElements.get(tagName)) {
        customElements.define(tagName, Radio);
      }
      break;
  } });
}
defineCustomElement$1();

const GrRadio = Radio;
const defineCustomElement = defineCustomElement$1;

export { GrRadio, defineCustomElement };

//# sourceMappingURL=gr-radio.js.map