/*!
 * (C) PAQT.com B.V. https://paqt.com - MIT License
 */
import { proxyCustomElement, HTMLElement, createEvent, h, Host } from '@stencil/core/internal/client';

const tabCss = ":host{display:inline-block;--color:var(--gr-color-dark);--color-focused:var(--gr-color-primary-contrast);--color-disabled:var(--gr-color-medium);display:inline-flex;align-items:center;font-family:var(--gr-font-family);font-size:var(--gr-font-size-medium);font-weight:var(--gr-font-weight-normal);color:var(--color);padding:var(--gr-spacing-x-small) var(--gr-spacing-medium);white-space:nowrap;user-select:none;cursor:pointer;transition:500ms box-shadow, 500ms color}:host:hover:not(.tab-disabled){color:var(--color-disabled)}:host(.tab-focused:not(.tab-disabled)){outline:none}:host(.tab-active:not(.tab-disabled)){color:var(--gr-color-primary)}:host(.tab-disabled){opacity:0.5;cursor:not-allowed}";

const Tab = /*@__PURE__*/ proxyCustomElement(class extends HTMLElement {
  constructor() {
    super();
    this.__registerHost();
    this.__attachShadow();
    this.grFocus = createEvent(this, "gr-focus", 7);
    this.grBlur = createEvent(this, "gr-blur", 7);
    this.hasFocus = false;
    this.panel = '';
    this.active = false;
    this.disabled = false;
  }
  connectedCallback() {
    this.onBlur = this.onBlur.bind(this);
    this.onFocus = this.onFocus.bind(this);
  }
  /** Sets focus on the tab. */
  async setFocus() {
    this.el.focus();
  }
  /** Removes focus from the tab. */
  async removeFocus() {
    this.el.blur();
  }
  onBlur() {
    this.hasFocus = false;
    this.grBlur.emit();
  }
  onFocus() {
    this.hasFocus = true;
    this.grFocus.emit();
  }
  render() {
    const { active, disabled } = this;
    return (h(Host, { class: {
        tab: true,
        'tab-active': active,
        'tab-disabled': disabled,
        'tab-focused': this.hasFocus,
      }, role: "tab", "aria-disabled": disabled ? 'true' : 'false', "aria-selected": active ? 'true' : 'false', tabIndex: disabled || !active ? 0 : null, onFocus: this.onFocus, onBlur: this.onBlur, slot: "nav" }, h("slot", null)));
  }
  get el() { return this; }
  static get style() { return tabCss; }
}, [1, "gr-tab", {
    "panel": [513],
    "active": [516],
    "disabled": [516],
    "hasFocus": [32],
    "setFocus": [64],
    "removeFocus": [64]
  }]);
function defineCustomElement$1() {
  if (typeof customElements === "undefined") {
    return;
  }
  const components = ["gr-tab"];
  components.forEach(tagName => { switch (tagName) {
    case "gr-tab":
      if (!customElements.get(tagName)) {
        customElements.define(tagName, Tab);
      }
      break;
  } });
}
defineCustomElement$1();

const GrTab = Tab;
const defineCustomElement = defineCustomElement$1;

export { GrTab, defineCustomElement };

//# sourceMappingURL=gr-tab.js.map